@@ -15,6 +15,7 @@
 import functools
 import logging
 import os
+import time
 
 from diskimage_builder.block_device.exception \
     import BlockDeviceSetupException
@@ -95,18 +96,35 @@ class MountPointNode(NodeBase):
 
     def umount(self):
         logger.info("Called for [%s]", self.name)
-        # Before calling umount call 'fstrim' on the mounted file
-        # system.  This discards unused blocks from the mounted
+        # Before calling umount, call 'fstrim' on suitable mounted
+        # file systems.  This discards unused blocks from the mounted
         # file system and therefore decreases the resulting image
         # size.
-        # A race condition can occur when trying to fstrim immediately after
-        # deleting a file resulting in that free space not being reclaimed.
-        # Calling sync before fstrim is a workaround for this behaviour.
+        #
+        # A race condition can occur when trying to fstrim immediately
+        # after deleting a file resulting in that free space not being
+        # reclaimed.  Calling sync before fstrim is a workaround for
+        # this behaviour.
         # https://lists.gnu.org/archive/html/qemu-devel/2014-03/msg02978.html
         exec_sudo(["sync"])
-        exec_sudo(["fstrim", "--verbose",
-                   self.state['mount'][self.mount_point]['path']])
-        exec_sudo(["umount", self.state['mount'][self.mount_point]['path']])
+        if self.state['filesys'][self.base]['fstype'] != 'vfat':
+            exec_sudo(["fstrim", "--verbose",
+                       self.state['mount'][self.mount_point]['path']])
+        # Even 'fstrim' call sometimes don't solve the issue with 'busy'
+        # filesystem, so we need to catch the exception and repeat unount.
+        mount_point = self.state['mount'][self.mount_point]['path']
+        catch = None
+        for try_cnt in range(10, 1, -1):
+            try:
+                exec_sudo(["umount", mount_point])
+                return
+            except BlockDeviceSetupException as e:
+                catch = e
+                logger.error("umount failed (%s)", e.output.strip())
+                time.sleep(3)
+
+        logger.debug("Gave up trying to umount [%s]", mount_point)
+        raise catch
 
     def delete(self):
         self.umount()
